'''help functions for machine learning
'''

from os import listdir
from os.path import isfile, join, splitext
import cv2
import numpy as np


def get_png_files_from_folder(folder):
    '''get file names from give folder
    '''
    onlyfiles = [f for f in listdir(folder) if isfile(join(folder, f))]
    png_files = [join(folder, f) for f in onlyfiles if splitext(f)[-1] == '.png']
    return png_files


def mirror_images(images):
    new_images = []
    for img in images:
        new_images.append(img)
        img_flip = cv2.flip(img, 1)
        new_images.append(img_flip)
    return new_images


def scale_brightness(images, gray, scales=[0.7, 1.3]):
    new_images = []
    for img in images:
        new_images.append(img)
        for s in scales:
            if gray:
                img_scale = img * s
                img_scale[img_scale > 254] = 254
            else:
                img_scale = np.asarray(img)
                y = img_scale[:, 0]
                y *= s
                y[y > 254] = 254
            img_scale = img_scale.astype(np.uint8)
            new_images.append(img_scale)
    return new_images


def shift_images(images, shift=[10]):
    new_images = []
    for img in images:
        new_images.append(img)
        for s in shift:
            img_shift0 = img[s:, s:]
            img_shift1 = img[s:, :-s]
            img_shift2 = img[:-s, :-s]
            img_shift3 = img[:-s, s:]
            new_images.append(img_shift0)
            new_images.append(img_shift1)
            new_images.append(img_shift2)
            new_images.append(img_shift3)
    return new_images


def load_images(png_files, labels, sample_shape, gray=True):
    '''load a list of images, and pack with labels for machine learning
    @param png_files: list of filenames
    @param labels: list of label for each file
    @param sample_shape: shape of input image, e.g. output image size
    @param gray: if true convert to gray image
    @return data of images: [[data ... label], ...]
    '''
    images = []
    for i, f in enumerate(png_files):
        img = cv2.imread(f, cv2.CV_LOAD_IMAGE_UNCHANGED)
        if gray:
            if  len(img) == 3:
                img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
        else:
            img = cv2.cvtColor(img, cv2.COLOR_BGR2YCR_CB)

        imgs = [img]
        if labels[i]:
            imgs = mirror_images(imgs)
            imgs = scale_brightness(imgs, gray)
            imgs = shift_images(imgs)

        for img in imgs:
            img = cv2.resize(img, sample_shape, interpolation=cv2.INTER_AREA)
            img = np.hstack((img.flatten(), labels[i])).astype(np.uint8)
            images.append(img)
    images = np.asarray(images)
    return images


def create_image_wall(images, sample_shape, columns=32, width=1024, mask=None, mask_color=(0, 255, 0)):
    '''put all images in one image for visualization
    '''
    rows = len(images) // columns + 1
    size_x = width // columns
    size_y = size_x
    gray = images.ndim == 2 or images.shape[-1] == 1

    if gray:
        image_show = np.zeros((rows * size_y, width), dtype=np.uint8)
    else:
        image_show = np.zeros((rows * size_y, width, 3), dtype=np.uint8)
    for y in range(rows):
        for x in range(0, columns):
            idx = y * columns + x
            if idx >= len(images):
                break
            image = images[idx]
            if gray:
                image = image.reshape(sample_shape[0], sample_shape[1])
            else:
                image = image.reshape(sample_shape[0], sample_shape[1], 3)
            thumbnail = cv2.resize(image, (size_x, size_y), interpolation=cv2.INTER_CUBIC)
            if mask is not None and mask[idx]:
                cv2.rectangle(thumbnail, (1, 1), (size_x - 1, size_y - 1), color=mask_color, thickness=2)
            image_show[y * size_y:y * size_y + size_y, x * size_x :x * size_x + size_x] = thumbnail
    if not gray:
        image_show = cv2.cvtColor(image_show, cv2.COLOR_YCR_CB2RGB)
    return image_show
